package de.duehl.algorithm.backtracking.eightqueens;

import de.duehl.algorithm.backtracking.Backtracking;
import de.duehl.basics.text.NumberString;

public class EightQueenSolver {

    private final EightQueens eightQueens;

    private final Backtracking<EightQueens, EightQueensStep> backtracking;

    private final int dimension;

    public EightQueenSolver(int dimension) {
        this.dimension = dimension;

        eightQueens = new EightQueens(dimension);
        backtracking = new Backtracking<>();

        backtracking.informAfterXSteps(100000);
    }

    public void create() {
        System.out.println("Lösung des Damenproblems in Dimesion  " + dimension + " läuft...");

        boolean hasSolution = backtracking.findSolution(eightQueens);

        System.out.println("Anzahl rekursiver Aufrufe      : "
                + NumberString.taupu(backtracking.getRecursiveCount()));
        System.out.println("Anzahl versuchter Schritte     : "
                + NumberString.taupu(backtracking.getTriedStepsCount()));
        System.out.println("Anzahl zurückgenommene Schritte: "
                + NumberString.taupu(backtracking.getBacktrackingCount()));
        System.out.println("Laufzeit                       : " + backtracking.getRuntime());

        if (hasSolution) {
            System.out.println("Lösung:\n" + eightQueens.toString());
        }
        else {
            System.out.println("Es gibt keine Lösung.");
        }
    }

    public static void main(String[] args) {
        new EightQueenSolver(8).create();
    }

}
