package de.duehl.algorithm.backtracking.subsets;

import java.util.ArrayList;
import java.util.List;

import de.duehl.algorithm.backtracking.AlgorithmState;

public class Subset implements AlgorithmState<SubsetStep> {

    private final List<List<Integer>> possibleIndices;

    private final List<Integer> indices;

    private final int subsetSize;

    private boolean chooseNextStepWasSuccessfull;

    public Subset(List<List<Integer>> possibleIndices) {
        this.possibleIndices = possibleIndices;
        subsetSize = possibleIndices.size();

        indices = new ArrayList<>();
    }

    @Override
    public boolean isVaild() {
        int lastIndex = Integer.MIN_VALUE;

        for (int index : indices) {
            if (index <= lastIndex) {
                return false;
            }
            lastIndex = index;
        }

        return true;
    }

    @Override
    public boolean isComplete() {
        return indices.size() == subsetSize;
    }

    @Override
    public void forward(SubsetStep step) {
        indices.add(Integer.valueOf(step.getIndex()));
    }

    @Override
    public void backward(SubsetStep step) {
        indices.remove(Integer.valueOf(step.getIndex()));
    }

    @Override
    public SubsetStep chooseNextStep(List<SubsetStep> badSteps) {
        chooseNextStepWasSuccessfull = true;

        int actualIndex = badSteps.size() - 1;
        List<Integer> nextIndices = possibleIndices.get(actualIndex + 1);

        for (int index : nextIndices) {
            SubsetStep nextStep = new SubsetStep(index);
            if (!badSteps.contains(nextStep)) {
                return nextStep;
            }
        }

        /*
         * Im Prinzip müsste man alle möglichen Lösungen berechnen und danach dann die mit dem
         * kleinsten Abstand von erstem zu letztem Abstand auswählen.
         */


        chooseNextStepWasSuccessfull = false;
        return null;
    }

    @Override
    public boolean chooseNextStepWasSuccessfull() {
        return chooseNextStepWasSuccessfull;
    }

    @Override
    public String toString() {
        return "Subset [indices=" + indices + ", subsetSize=" + subsetSize + "]";
    }


}
