package de.duehl.basics.io.textfile;

/*
 * Copyright 2019 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

import java.util.ArrayList;
import java.util.List;

import de.duehl.basics.text.Text;

/**
 * Diese Klasse liest eine Liste mit den Texten aus den Zeilen einer Textdatei ein.
 *
 * Die Zeilen werden getrimmt. Dabei werden leere Zeilen und solche, die mit dem Kommentarzeichen
 * (ggf., nach Leerraum) überlesen. Kommt in einer Datenzeile weiter hinten das Kommentarzeichen
 * vor, so wird ab dort abgeschnitten und getrimmt.
 *
 * Der Zeileninhalt wird dann als Komma-getrennte Liste verarbeitet.
 *
 * @version 1.01     2019-03-21
 * @author Christian Dühl
 */

public class ListsFromTextFileReader {

    private final StringsFromTextFileReader reader;

    /** Gibt an, ob der Reader ruhig sein soll. */
    private boolean quiet;


    public ListsFromTextFileReader(StringsFromTextFileReader reader) {
        this.reader = reader;

        quiet = false;
    }

    /** Legt fest, dass der Reader ruhig sein soll. */
    public void beQuiet( ) {
        quiet = true;
    }

    /** Liest die Listen aus der Datei ein. */
    public List<List<String>> read() {
        if (quiet) {
            reader.beQuiet();
        }
        List<String> lines = reader.read();
        return aplitLines(lines);
    }

    private List<List<String>> aplitLines(List<String> lines) {
        List<List<String>> listOfLists = new ArrayList<>();

        for (String line : lines) {
            List<String> list = Text.splitByKomma(line);
            listOfLists.add(list);
        }

        return listOfLists;
    }

}
