package de.duehl.basics.system;

/*
 * Copyright 2018 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

import static org.junit.Assert.*;

import org.junit.Test;

import de.duehl.basics.text.Text;

public class ExceptionHelperTest {

    private static final String LOWER_STACK_TRACE_PART = ""
            + "     2: java.base/jdk.internal.reflect.NativeMethodAccessorImpl.invoke0(Native Method)" + Text.LINE_BREAK
            + "     3: java.base/jdk.internal.reflect.NativeMethodAccessorImpl.invoke(NativeMethodAccessorImpl.java:62)" + Text.LINE_BREAK
            + "     4: java.base/jdk.internal.reflect.DelegatingMethodAccessorImpl.invoke(DelegatingMethodAccessorImpl.java:43)" + Text.LINE_BREAK
            + "     5: java.base/java.lang.reflect.Method.invoke(Method.java:567)" + Text.LINE_BREAK
            + "     6: org.junit.runners.model.FrameworkMethod$1.runReflectiveCall(FrameworkMethod.java:50)" + Text.LINE_BREAK
            + "     7: org.junit.internal.runners.model.ReflectiveCallable.run(ReflectiveCallable.java:12)" + Text.LINE_BREAK
            + "     8: org.junit.runners.model.FrameworkMethod.invokeExplosively(FrameworkMethod.java:47)" + Text.LINE_BREAK
            + "     9: org.junit.internal.runners.statements.InvokeMethod.evaluate(InvokeMethod.java:17)" + Text.LINE_BREAK
            + "    10: org.junit.runners.ParentRunner.runLeaf(ParentRunner.java:325)" + Text.LINE_BREAK
            + "    11: org.junit.runners.BlockJUnit4ClassRunner.runChild(BlockJUnit4ClassRunner.java:78)" + Text.LINE_BREAK
            + "    12: org.junit.runners.BlockJUnit4ClassRunner.runChild(BlockJUnit4ClassRunner.java:57)" + Text.LINE_BREAK
            + "    13: org.junit.runners.ParentRunner$3.run(ParentRunner.java:290)" + Text.LINE_BREAK
            + "    14: org.junit.runners.ParentRunner$1.schedule(ParentRunner.java:71)" + Text.LINE_BREAK
            + "    15: org.junit.runners.ParentRunner.runChildren(ParentRunner.java:288)" + Text.LINE_BREAK
            + "    16: org.junit.runners.ParentRunner.access$000(ParentRunner.java:58)" + Text.LINE_BREAK
            + "    17: org.junit.runners.ParentRunner$2.evaluate(ParentRunner.java:268)" + Text.LINE_BREAK
            + "    18: org.junit.runners.ParentRunner.run(ParentRunner.java:363)" + Text.LINE_BREAK
            + "    19: org.eclipse.jdt.internal.junit4.runner.JUnit4TestReference.run(JUnit4TestReference.java:89)" + Text.LINE_BREAK
            + "    20: org.eclipse.jdt.internal.junit.runner.TestExecution.run(TestExecution.java:41)" + Text.LINE_BREAK
            + "    21: org.eclipse.jdt.internal.junit.runner.RemoteTestRunner.runTests(RemoteTestRunner.java:541)" + Text.LINE_BREAK
            + "    22: org.eclipse.jdt.internal.junit.runner.RemoteTestRunner.runTests(RemoteTestRunner.java:763)" + Text.LINE_BREAK
            + "    23: org.eclipse.jdt.internal.junit.runner.RemoteTestRunner.run(RemoteTestRunner.java:463)" + Text.LINE_BREAK
            + "    24: org.eclipse.jdt.internal.junit.runner.RemoteTestRunner.main(RemoteTestRunner.java:209)" + Text.LINE_BREAK;

    private static final String LOWER_STACK_TRACE_PART_HTML = ""
            + "    <li>java.base/jdk.internal.reflect.NativeMethodAccessorImpl.invoke0(Native Method)</li>" + Text.LINE_BREAK
            + "    <li>java.base/jdk.internal.reflect.NativeMethodAccessorImpl.invoke(NativeMethodAccessorImpl.java:62)</li>" + Text.LINE_BREAK
            + "    <li>java.base/jdk.internal.reflect.DelegatingMethodAccessorImpl.invoke(DelegatingMethodAccessorImpl.java:43)</li>" + Text.LINE_BREAK
            + "    <li>java.base/java.lang.reflect.Method.invoke(Method.java:567)</li>" + Text.LINE_BREAK
            + "    <li>org.junit.runners.model.FrameworkMethod$1.runReflectiveCall(FrameworkMethod.java:50)</li>" + Text.LINE_BREAK
            + "    <li>org.junit.internal.runners.model.ReflectiveCallable.run(ReflectiveCallable.java:12)</li>" + Text.LINE_BREAK
            + "    <li>org.junit.runners.model.FrameworkMethod.invokeExplosively(FrameworkMethod.java:47)</li>" + Text.LINE_BREAK
            + "    <li>org.junit.internal.runners.statements.InvokeMethod.evaluate(InvokeMethod.java:17)</li>" + Text.LINE_BREAK
            + "    <li>org.junit.runners.ParentRunner.runLeaf(ParentRunner.java:325)</li>" + Text.LINE_BREAK
            + "    <li>org.junit.runners.BlockJUnit4ClassRunner.runChild(BlockJUnit4ClassRunner.java:78)</li>" + Text.LINE_BREAK
            + "    <li>org.junit.runners.BlockJUnit4ClassRunner.runChild(BlockJUnit4ClassRunner.java:57)</li>" + Text.LINE_BREAK
            + "    <li>org.junit.runners.ParentRunner$3.run(ParentRunner.java:290)</li>" + Text.LINE_BREAK
            + "    <li>org.junit.runners.ParentRunner$1.schedule(ParentRunner.java:71)</li>" + Text.LINE_BREAK
            + "    <li>org.junit.runners.ParentRunner.runChildren(ParentRunner.java:288)</li>" + Text.LINE_BREAK
            + "    <li>org.junit.runners.ParentRunner.access$000(ParentRunner.java:58)</li>" + Text.LINE_BREAK
            + "    <li>org.junit.runners.ParentRunner$2.evaluate(ParentRunner.java:268)</li>" + Text.LINE_BREAK
            + "    <li>org.junit.runners.ParentRunner.run(ParentRunner.java:363)</li>" + Text.LINE_BREAK
            + "    <li>org.eclipse.jdt.internal.junit4.runner.JUnit4TestReference.run(JUnit4TestReference.java:89)</li>" + Text.LINE_BREAK
            + "    <li>org.eclipse.jdt.internal.junit.runner.TestExecution.run(TestExecution.java:41)</li>" + Text.LINE_BREAK
            + "    <li>org.eclipse.jdt.internal.junit.runner.RemoteTestRunner.runTests(RemoteTestRunner.java:541)</li>" + Text.LINE_BREAK
            + "    <li>org.eclipse.jdt.internal.junit.runner.RemoteTestRunner.runTests(RemoteTestRunner.java:763)</li>" + Text.LINE_BREAK
            + "    <li>org.eclipse.jdt.internal.junit.runner.RemoteTestRunner.run(RemoteTestRunner.java:463)</li>" + Text.LINE_BREAK
            + "    <li>org.eclipse.jdt.internal.junit.runner.RemoteTestRunner.main(RemoteTestRunner.java:209)</li>" + Text.LINE_BREAK;

    @Test
    public void getDescriptionWithStackTraceNormalException() {
        Exception exception = new Exception("Da ist was passiert!");
        String actual = ExceptionHelper.getDescriptionWithStackTrace(exception);
        String expected = "Klassenname: java.lang.Exception" + Text.LINE_BREAK
                + "Nachricht  : Da ist was passiert!" + Text.LINE_BREAK
                + "Stack-Trace: " + Text.LINE_BREAK
                + "     1: de.duehl.basics.system.ExceptionHelperTest.getDescriptionWithStackTraceNormalException(ExceptionHelperTest.java:74)" + Text.LINE_BREAK
                + LOWER_STACK_TRACE_PART;
        assertEquals(expected, actual);
    }

    @Test
    public void getDescriptionWithStackTraceExceptionWithCause() {
        Exception exception = new Exception("Da ist was passiert!",
                new Exception("Da ist was anderes passiert!"));
        String actual = ExceptionHelper.getDescriptionWithStackTrace(exception);
        String expected = "Klassenname: java.lang.Exception" + Text.LINE_BREAK
                + "Nachricht  : Da ist was passiert!" + Text.LINE_BREAK
                + "Stack-Trace: " + Text.LINE_BREAK
                + "     1: de.duehl.basics.system.ExceptionHelperTest.getDescriptionWithStackTraceExceptionWithCause(ExceptionHelperTest.java:86)" + Text.LINE_BREAK
                + LOWER_STACK_TRACE_PART
                + "Cause      : " + Text.LINE_BREAK
                + "Klassenname: java.lang.Exception" + Text.LINE_BREAK
                + "Nachricht  : Da ist was anderes passiert!" + Text.LINE_BREAK
                + "Stack-Trace: " + Text.LINE_BREAK
                + "     1: de.duehl.basics.system.ExceptionHelperTest.getDescriptionWithStackTraceExceptionWithCause(ExceptionHelperTest.java:87)" + Text.LINE_BREAK
                + LOWER_STACK_TRACE_PART;
        assertEquals(expected, actual);
    }

    @Test
    public void getDescriptionWithStackTraceExceptionWithTwoCauseses() {
        Exception exception = new Exception("Da ist was passiert!",
                new Exception("Da ist was anderes passiert!",
                        new Exception("Da ist was noch etwas anderes passiert!")));
        String actual = ExceptionHelper.getDescriptionWithStackTrace(exception);
        String expected = "Klassenname: java.lang.Exception" + Text.LINE_BREAK
                + "Nachricht  : Da ist was passiert!" + Text.LINE_BREAK
                + "Stack-Trace: " + Text.LINE_BREAK
                + "     1: de.duehl.basics.system.ExceptionHelperTest.getDescriptionWithStackTraceExceptionWithTwoCauseses(ExceptionHelperTest.java:105)" + Text.LINE_BREAK
                + LOWER_STACK_TRACE_PART
                + "Cause      : " + Text.LINE_BREAK
                + "Klassenname: java.lang.Exception" + Text.LINE_BREAK
                + "Nachricht  : Da ist was anderes passiert!" + Text.LINE_BREAK
                + "Stack-Trace: " + Text.LINE_BREAK
                + "     1: de.duehl.basics.system.ExceptionHelperTest.getDescriptionWithStackTraceExceptionWithTwoCauseses(ExceptionHelperTest.java:106)" + Text.LINE_BREAK
                + LOWER_STACK_TRACE_PART
                + "Cause      : " + Text.LINE_BREAK
                + "Klassenname: java.lang.Exception" + Text.LINE_BREAK
                + "Nachricht  : Da ist was noch etwas anderes passiert!" + Text.LINE_BREAK
                + "Stack-Trace: " + Text.LINE_BREAK
                + "     1: de.duehl.basics.system.ExceptionHelperTest.getDescriptionWithStackTraceExceptionWithTwoCauseses(ExceptionHelperTest.java:107)" + Text.LINE_BREAK
                + LOWER_STACK_TRACE_PART;
        assertEquals(expected, actual);
    }

    @Test
    public void getExceptionNameAndMessageNormalException() {
        Exception exception = new Exception("Da ist was passiert!");
        String actual = ExceptionHelper.getExceptionNameAndMessage(exception);
        String expected = "Klasse der Ausnahme: java.lang.Exception, "
                + "Fehlermeldung der Ausnahme: Da ist was passiert!";
        assertEquals(expected, actual);
    }

    @Test
    public void getExceptionNameAndMessageExceptionWithCause() {
        Exception exception = new Exception("Da ist was passiert!",
                new Exception("Da ist was anderes passiert!"));
        String actual = ExceptionHelper.getExceptionNameAndMessage(exception);
        String expected = "Klasse der Ausnahme: java.lang.Exception, "
                + "Fehlermeldung der Ausnahme: Da ist was passiert! "
                + "(caused by: Klasse der Ausnahme: java.lang.Exception, "
                + "Fehlermeldung der Ausnahme: Da ist was anderes passiert!)";
        assertEquals(expected, actual);
    }

    @Test
    public void getExceptionNameAndMessageExceptionWithTwoCauseses() {
        Exception exception = new Exception("Da ist was passiert!",
                new Exception("Da ist was anderes passiert!",
                        new Exception("Da ist was noch etwas anderes passiert!")));
        String actual = ExceptionHelper.getExceptionNameAndMessage(exception);
        String expected = "Klasse der Ausnahme: java.lang.Exception, "
                + "Fehlermeldung der Ausnahme: Da ist was passiert! "
                + "(caused by: Klasse der Ausnahme: java.lang.Exception, "
                + "Fehlermeldung der Ausnahme: Da ist was anderes passiert! "
                + "(caused by: Klasse der Ausnahme: java.lang.Exception, "
                + "Fehlermeldung der Ausnahme: Da ist was noch etwas anderes passiert!))";
        assertEquals(expected, actual);
    }

    @Test
    public void getExceptionNameAndMessageExceptionWithThreeCauseses() {
        Exception exception = new Exception("Da ist was passiert!",
                new Exception("Da ist was anderes passiert!",
                        new Exception("Da ist was noch etwas anderes passiert!",
                                new Exception("Da ist was noch etwas ganz anderes passiert!"))));
        String actual = ExceptionHelper.getExceptionNameAndMessage(exception);
        String expected = "Klasse der Ausnahme: java.lang.Exception, "
                + "Fehlermeldung der Ausnahme: Da ist was passiert! "
                + "(caused by: Klasse der Ausnahme: java.lang.Exception, "
                + "Fehlermeldung der Ausnahme: Da ist was anderes passiert! "
                + "(caused by: Klasse der Ausnahme: java.lang.Exception, "
                + "Fehlermeldung der Ausnahme: Da ist was noch etwas anderes passiert! "
                + "(caused by: Klasse der Ausnahme: java.lang.Exception, "
                + "Fehlermeldung der Ausnahme: Da ist was noch etwas ganz anderes passiert!)))";
        assertEquals(expected, actual);
    }

    @Test
    public void getDescriptionWithStackTraceExceptionHtmlWithTwoCauseses() {
        Exception exception = new Exception("Da ist was passiert!",
                new Exception("Da ist was anderes passiert!",
                        new Exception("Da ist was noch etwas anderes passiert!")));
        String actual = ExceptionHelper.getDescriptionWithStackTraceHtml(exception);
        String expected = "<p>Klassenname: java.lang.Exception</p>" + Text.LINE_BREAK
                + "<p>Nachricht  : Da ist was passiert!</p>" + Text.LINE_BREAK
                + "<p>Stack-Trace:</p>" + Text.LINE_BREAK
                + "<ol>" + Text.LINE_BREAK
                + "    <li>de.duehl.basics.system.ExceptionHelperTest.getDescriptionWithStackTraceExceptionHtmlWithTwoCauseses(ExceptionHelperTest.java:185)</li>" + Text.LINE_BREAK
                + LOWER_STACK_TRACE_PART_HTML
                + "</ol>" + Text.LINE_BREAK
                + "<p>Cause      :</p>" + Text.LINE_BREAK
                + "<p>Klassenname: java.lang.Exception</p>" + Text.LINE_BREAK
                + "<p>Nachricht  : Da ist was anderes passiert!</p>" + Text.LINE_BREAK
                + "<p>Stack-Trace:</p>" + Text.LINE_BREAK
                + "<ol>" + Text.LINE_BREAK
                + "    <li>de.duehl.basics.system.ExceptionHelperTest.getDescriptionWithStackTraceExceptionHtmlWithTwoCauseses(ExceptionHelperTest.java:186)</li>" + Text.LINE_BREAK
                + LOWER_STACK_TRACE_PART_HTML
                + "</ol>" + Text.LINE_BREAK
                + "<p>Cause      :</p>" + Text.LINE_BREAK
                + "<p>Klassenname: java.lang.Exception</p>" + Text.LINE_BREAK
                + "<p>Nachricht  : Da ist was noch etwas anderes passiert!</p>" + Text.LINE_BREAK
                + "<p>Stack-Trace:</p>" + Text.LINE_BREAK
                + "<ol>" + Text.LINE_BREAK
                + "    <li>de.duehl.basics.system.ExceptionHelperTest.getDescriptionWithStackTraceExceptionHtmlWithTwoCauseses(ExceptionHelperTest.java:187)</li>" + Text.LINE_BREAK
                + LOWER_STACK_TRACE_PART_HTML
                + "</ol>" + Text.LINE_BREAK;
        assertEquals(expected, actual);
    }

}
