package de.duehl.math.graph.ged.ui;

import java.awt.image.BufferedImage;

import javax.swing.JFrame;

import de.duehl.math.graph.ged.graph.GedColor;
import de.duehl.math.graph.ged.graph.Graph;
import de.duehl.math.graph.ged.graph.edge.Edge;
import de.duehl.math.graph.ged.graph.edge.Edges;
import de.duehl.math.graph.ged.graph.vertex.Vertex;
import de.duehl.math.graph.ged.pixel.PixelPoint;
import de.duehl.math.graph.ged.ui.graphpanel.GraphPanel;
import de.duehl.basics.history.HistoryStateDisplayer;

/**
 * Dieses Interface steht für die Schnittstelle zur grafischen Oberfläche des Programms.
 *
 * @version 1.01     2019-07-02
 * @author Christian Dühl
 */

public interface Gui extends HistoryStateDisplayer {

    /** Getter für den graphPanel */
    GraphPanel getGraphPanel();

    /** Setzt den Änderungsmarker. */
    void setChanged();

    /**
     * Setzt den darzustellenden Graphen.
     *
     * @param graph
     *            darzustellender Graphen
     */
    void setGraph(Graph graph);

    /** Gibt den dargestellten Graphen zurück. */
    Graph getGraph();

    /**
     * Setzt den Statustext.
     *
     * @param statusText
     *            Anzuzeigender Text.
     */
    void setStatus(String string);

    /** Gibt den zugrundeliegenden JFrame zurück. */
    JFrame getFrame();

    /** Startet einen Prozess, der eine Weile braucht. Es wird der Waitcursor angezeigt. */
    void startLongTimeProcess(String message);

    /** Beendet einen Prozess, der eine Weile braucht. Es wird der normale Cursor angezeigt. */
    void endLongTimeProcess();

    /** Unterbricht eine ggf. begonnene Bearbeitung. */
    void interruptEditing();

    /** Fertigt einen Snapshot vom graphenPanel an. */
    BufferedImage getSnapshot();

    /** Zeigt dem Benutzer einen Fehler an. */
    void error(String string);

    /** Zeigt dem Benutzer einen Fehler an. */
    void error(String string, Exception exception);

    /** Wählt die Defaultfarbe für die Ecken aus. */
    GedColor chooseDefaultVertexColor();

    /** Wählt die Defaultfarbe für die Kanten aus. */
    GedColor chooseDefaultEdgeColor();

    /** Zeigt den Dialog zum Bearbeiten der Eckeneigenschaften an. */
    void showVertexPopupMenu(Vertex vertex, PixelPoint coords);

    /** Zeigt den Dialog zum Bearbeiten der Kanteneigenschaften an. */
    void showEdgePopupMenu(Edge edge, PixelPoint coords);

    /** Zeigt einen Textdialog mit dem übergebenen Titel und Inhalt an. */
    void showTextDialog(String title, String content);

    /**
     * Bestimmt aus einer Menge von Kanten, welche davon gelöscht werden sollen
     *
     * @param edgesAtCoords
     *            Menge mit den Kanten, die in der Nähe einer bestimmten Bildschirmkoordinate
     *            verlaufen.
     * @param coords
     *            Koordinaten
     * @return Menge der Kanten, die wirklich gelöscht werden sollen.
     */
    Edges determineEdgesToDelete(Edges edgesAtCoords, PixelPoint coords);

    /**
     * Zeigt den Dateinamen im Titel an.
     *
     * @param filename
     *            Dateiname
     */
    void showFileName(String filename);

    /**
     * Fragt den Benutzer nach dem Defaultradius für die Ecken.
     *
     * @param oldValue
     *            Alter Wert.
     * @param description
     *            Beschreibung der Einheit, dessen Größe geändert werden soll ("Radius").
     */
    int askUserForSizeValue(int oldValue, String description);

    /** Zeichnet den Graphpanel neu. */
    void repaintGraphPanel();

    /** Zeigt die Größe des GraphPanels an. */
    void showGraphPanelSize(int width, int height);

    /**
     * Setzt die Größe der Canvas.
     *
     * @param width
     *            neue Breite
     * @param height
     *            neue Höhe
     */
    void setViewSize(int width, int height);

    /** Fragt den Benutzer, ob das Programm beendet werden soll. */
    boolean askUserToQuit();

    void setVisible(boolean visible);

    String askUserToOpenAsGed3();
    String askUserToOpenAsGed4();
    String askUserToSaveAsGed3();
    String askUserToSaveAsGed4(String oldFilename);
    String askUserAboutNameForScreenshot();
    String askUserToOpenBackgroundPicture();

    /** Zeigt den Über-Dialog an. */
    void showAbout();

    /** Zeigt die Hilfe an. */
    void showHelp();

    /** Zeigt die offenen Punkte an. */
    void showTodo();

    /** Zeigt die Veränderungen an. */
    void showChanges();

    /** Schaltet den Graphen zwischen gerichtet und ungerichtet um. */
    void switchDirectedGraph(boolean directed);

}
