package de.duehl.math.graph.ged.ui.elements;

import java.awt.Color;
import java.awt.Component;

import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JPanel;

import de.duehl.math.graph.ged.graph.GedColor;
import de.duehl.math.graph.ged.ui.GedGuiTools;
import de.duehl.math.graph.ged.ui.elements.color.ColorSetter;

/**
 * Diese Klasse stellt für den Ecken- und Kanten-Dialog Eingabefelder für Wahlweise zu verwendende
 * Daten mit Eingabefeld des Wertes, X- und Y-Abstand und eine Farbwahl mit Default-Schalter dar.
 *
 * @version 1.01     2019-05-27
 * @author Christian Dühl
 */

public class SwitchableValueDistanceAndColorChooser {

    private JPanel panel;

    /** Gibt an, ob die hier einstellbaren Werte verwendet werden. */
    private final boolean uses;

    /** Text für die CheckBox, die angibt, ob die hier einstellbaren Werte verwendet werden. */
    private final String textForMainCheckBox;

    /** Text vor dem Eingabefeld für den Wert. */
    private final String titleOfValueInputField;

    /** Gibt an, ob zu Beginn die Default-Farbe verwendet wird. */
    private final boolean hasDefaultColor;

    /** Text vor dem Eingabefeld zum Abstand in X-Richtung. */
    private final String textForXDistance;

    /** Text vor dem Eingabefeld zum Abstand in Y-Richtung. */
    private final String textForYDistance;

    /** Text der auf der Checkbox zum an und Ausschalten des Default-Zustands angezeigt wird. */
    private final String textForToggleButton;

    /** Text der als Titel vor dem Farbbutton angezeigt wird. */
    private final String textForColorButtonTitle;

    /** Text der als Titel des Dialogs zur Farbauswahl angezeigt wird. */
    private final String textForColorChooserDialogTitle;

    /** Farbe des Objekts zum Zeitpunkt des Aufrufs. */
    private final Color actualSwingColor;

    /** Default-Farbe des Objekts. */
    private final GedColor defaultGedColor;

    /** Setzt die Farbe im Objekt. */
    private final ColorSetter colorSetter;

    /** Rahmen von GED zur relativen Positionierung des Farbauswahldialogs. */
    private final JFrame frame;

    private JCheckBox mainCheckBox;
    private ValueDistanceAndColorChooser chooser;

    /**
     * Konstruktor.
     *
     * @param uses
     *            Gibt an, ob die hier einstellbaren Werte verwendet werden.
     * @param textForMainCheckBox
     *            Text für die CheckBox, die angibt, ob die hier einstellbaren Werte verwendet
     *            werden.
     * @param titleOfValueInputField
     *            Text vor dem Eingabefeld für den Wert.
     * @param hasDefaultColor
     *            Gibt an, ob zu Beginn die Default-Farbe verwendet wird.
     * @param textForXDistance
     *            Text vor dem Eingabefeld zum Abstand in X-Richtung.
     * @param textForYDistance
     *            Text vor dem Eingabefeld zum Abstand in Y-Richtung.
     * @param textForToggleButton
     *            Text der auf der Checkbox zum an und Ausschalten des Default-Zustands angezeigt
     *            wird.
     * @param textForColorButtonTitle
     *            Text der als Titel vor dem Farbbutton angezeigt wird.
     * @param textForColorChooserDialogTitle
     *            Text der als Titel des Dialogs zur Farbauswahl angezeigt wird.
     * @param actualSwingColor
     *            Farbe des Objekts zum Zeitpunkt des Aufrufs.
     * @param defaultGedColor
     *            Default-Farbe des Objekts.
     * @param colorSetter
     *            Setzt die Farbe im Objekt.
     * @param frame
     *            Rahmen von GED zur relativen Positionierung des Farbauswahldialogs.
     */
    public SwitchableValueDistanceAndColorChooser(boolean uses, String textForMainCheckBox,
            String titleOfValueInputField, boolean hasDefaultColor, String textForXDistance,
            String textForYDistance, String textForToggleButton, String textForColorButtonTitle,
            String textForColorChooserDialogTitle, Color actualSwingColor, GedColor defaultGedColor,
            ColorSetter colorSetter, JFrame frame) {
        this.uses = uses;
        this.textForMainCheckBox = textForMainCheckBox;
        this.titleOfValueInputField = titleOfValueInputField;
        this.hasDefaultColor = hasDefaultColor;
        this.textForXDistance = textForXDistance;
        this.textForYDistance = textForYDistance;
        this.textForToggleButton = textForToggleButton;
        this.textForColorButtonTitle = textForColorButtonTitle;
        this.textForColorChooserDialogTitle = textForColorChooserDialogTitle;
        this.actualSwingColor = actualSwingColor;
        this.defaultGedColor = defaultGedColor;
        this.colorSetter = colorSetter;
        this.frame = frame;

        panel = createValueDistanceAndColor();
        setEnabled(uses);
    }

    private JPanel createValueDistanceAndColor() {
        JPanel panel = new JPanel();
        panel.setLayout(GedGuiTools.createVerticalComponentLayout());

        panel.add(createMainCheckBox());
        panel.add(createValueDistanceAndColorChooser());

        return panel;
    }

    private Component createMainCheckBox() {
        mainCheckBox = new JCheckBox(textForMainCheckBox);
        mainCheckBox.setSelected(uses);
        mainCheckBox.addActionListener(e -> checkBoxToggled());

        return mainCheckBox;
    }

    private void checkBoxToggled() {
        boolean enabled = mainCheckBox.isSelected();
        chooser.setEnabled(enabled);
    }

    private Component createValueDistanceAndColorChooser() {
        chooser = new NumericValueDistanceAndColorChooser(
                titleOfValueInputField,
                hasDefaultColor,
                textForXDistance,
                textForYDistance,
                textForToggleButton,
                textForColorButtonTitle,
                textForColorChooserDialogTitle,
                actualSwingColor,
                defaultGedColor,
                colorSetter,
                frame);

        return chooser.getPanel();
    }

    public void setStartValue(String startValue) {
        chooser.setStartValue(startValue);
    }

    public void setXDistance(int xDistance) {
        chooser.setXDistance(xDistance);
    }

    public void setYDistance(int yDistance) {
        chooser.setYDistance(yDistance);
    }

    public JPanel getPanel() {
        return panel;
    }

    public void setEnabled(boolean enabled) {
        //mainCheckBox.setEnabled(enabled); // dann kann man sie nicht mehr aktivieren!
        chooser.setEnabled(enabled);
    }

    public String getValue() {
        return chooser.getValue();
    }

    public String getXDistanceFieldText() {
        return chooser.getXDistanceFieldText();
    }

    public String getYDistanceFieldText() {
        return chooser.getYDistanceFieldText();
    }

    public boolean isDefaultColorSelected() {
        return chooser.isDefaultColorSelected();
    }

    public GedColor getSelectedColor() {
        return chooser.getSelectedColor();
    }

    public boolean isMainCheckBoxSelected() {
        return mainCheckBox.isSelected();
    }

    public void setDefaultButtonColor(GedColor gedColor) {
        chooser.setDefaultButtonColor(gedColor);
    }

    public void addChangeValueAction(Runnable runnable) {
        chooser.addChangeValueAction(runnable);
    }

    public void addChangeXDistanceAction(Runnable runnable) {
        chooser.addChangeXDistanceAction(runnable);
    }

    public void addChangeYDistanceAction(Runnable runnable) {
        chooser.addChangeYDistanceAction(runnable);
    }

}
