package de.duehl.math.graph.ged.graph.subGraph;

import de.duehl.math.graph.ged.graph.Graph;
import de.duehl.math.graph.ged.graph.edge.AdjacenceEntry;
import de.duehl.math.graph.ged.graph.vertex.Vertex;

/**
 * Diese Klasse erstellt Teilgraphen mit einer Kante.
 *
 * @version 1.02     2019-05-27
 * @author Christian Dühl
 */

public class EdgeSubGraphCreator extends SubgraphCreator {

    /**
     * Konstruktor.
     *
     * @param graph
     *            Graph zu dem Teilgraphen zu erstellen sind.
     * @param subGraphSize
     *            Größe der quadratischen Teilgraphen.
     */
    public EdgeSubGraphCreator(Graph graph, int subGraphSize) {
        super(graph, subGraphSize, subGraphSize);
    }

    /**
     * Konstruktor.
     *
     * @param graph
     *            Graph zu dem Teilgraphen zu erstellen sind.
     * @param subGraphSizeX
     *            Breite der Teilgraphen.
     * @param subGraphSizeY
     *            Höhe der Teilgraphen.
     */
    public EdgeSubGraphCreator(Graph graph, int subGraphSizeX, int subGraphSizeY) {
        super(graph, subGraphSizeX, subGraphSizeY);
    }

    /**
     * Erzeugt einen Teilgraphen der eine Kante anzeigt.
     *
     * @param startIndex
     *            Index der Startecke.
     * @param entry
     *            Daten der Kante.
     */
    public Graph createOneEdgeSubGraph(int startIndex, AdjacenceEntry entry) {
        return createOneEdgeSubGraph(graph.getVertex(startIndex), entry);
    }

    /**
     * Erzeugt einen Teilgraphen der eine Kante anzeigt.
     *
     * @param startVertex
     *            Startecke.
     * @param entry
     *            Daten der Kante.
     */
    public Graph createOneEdgeSubGraph(Vertex startVertex, AdjacenceEntry entry) {
        int endIndex = entry.getNeighbourIndex();
        Vertex endVertex = graph.getVertex(endIndex);

        /* Neue Ecken erzeugen: */
        Vertex subGraphStartVertex = new Vertex(startVertex);
        Vertex subGraphEndVertex = new Vertex(endVertex);

        /* Neue Kante erzeugen: */
        AdjacenceEntry subGraphEntry = new AdjacenceEntry(entry);
        subGraphEntry.setNeighbourIndex(1); // wichtig, nicht etwa newEndVertex.getIndex() !
                                            // der muss vor addEdge() stimmen!

        createSubGraphWithCopiedMeta();

        /* Ecken und Kanten einbauen: */
        subGraph.addVertex(subGraphStartVertex); // Index 0 - wird automatisch korrigiert
        subGraph.addVertex(subGraphEndVertex);   // Index 1 - wird automatisch korrigiert
        subGraph.addEdge(0, subGraphEntry);

        maximizeSubGraph(subGraph);

        return subGraph;
    }


}
