package de.duehl.math.graph.ged.pixel;

/**
 * Diese Klasse steht für einen zweidimensionalen Punkt in ganzen Zahlen, z.B. für Pixel.
 *
 * @version 1.01     2019-05-21
 * @author Christian Dühl
 */

public class PixelPoint {

    private int x;

    private int y;

    public PixelPoint() {
        this(0, 0);
    }

    public PixelPoint(int x, int y) {
        this.x = x;
        this.y = y;
    }

    public PixelPoint(PixelPoint that) {
        this(that.x, that.y);
    }

    public PixelPoint(double x, double y) {
        this((int) x, (int) y);
    }

    public PixelPoint(java.awt.Point point) {
        this(point.getX(), point.getY());
    }

    public PixelPoint(de.duehl.math.geometry.Point point) {
        this(point.getX(), point.getY());
    }

    public int getX() {
        return x;
    }

    public void setX(int x) {
        this.x = x;
    }

    public void setX(double x) {
        this.x = (int) x;
    }

    public int getY() {
        return y;
    }

    public void setY(int y) {
        this.y = y;
    }

    public void setY(double y) {
        this.y = (int) y;
    }

    public de.duehl.math.geometry.Point toMathPoint() {
        return new de.duehl.math.geometry.Point(x, y);
    }

    @Override
    public String toString() {
        return "PixelPoint [x=" + x + ", y=" + y + "]";
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + x;
        result = prime * result + y;
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        PixelPoint other = (PixelPoint) obj;
        if (x != other.x)
            return false;
        if (y != other.y)
            return false;
        return true;
    }

}
